@@ -20,14 +20,11 @@
 
 struct roots_cursor *roots_cursor_create(struct roots_seat *seat) {
 	struct roots_cursor *cursor = calloc(1, sizeof(struct roots_cursor));
-	if (!cursor) {
-		return NULL;
-	}
+	g_assert (cursor);
+
 	cursor->cursor = wlr_cursor_create();
-	if (!cursor->cursor) {
-		free(cursor);
-		return NULL;
-	}
+	g_assert (cursor->cursor);
+
 	cursor->default_xcursor = ROOTS_XCURSOR_DEFAULT;
 	return cursor;
 }
@@ -101,7 +98,7 @@ static void seat_view_deco_button(struct roots_seat_view *view, double sx,
 	}
 }
 
-static bool roots_handle_shell_reveal(struct wlr_surface *surface, double lx, double ly, double threshold) {
+static bool roots_handle_shell_reveal(struct wlr_surface *surface, double lx, double ly, int threshold) {
 	PhocServer *server = phoc_server_get_default ();
 	PhocDesktop *desktop = server->desktop;
 
@@ -156,10 +153,10 @@ static bool roots_handle_shell_reveal(struct wlr_surface *surface, double lx, do
 		}
 	}
 
-	if ((top    && ly <= output_box->y + threshold * output_box->height) ||
-			(bottom && ly >= output_box->y + (1.0 - threshold) * output_box->height - 1) ||
-			(left   && lx <= output_box->x + threshold * output_box->width) ||
-			(right  && lx >= output_box->x + (1.0 - threshold) * output_box->width - 1)) {
+	if ((top    && ly <= output_box->y + threshold) ||
+			(bottom && ly >= output_box->y + output_box->height - 1 - threshold) ||
+			(left   && lx <= output_box->x + threshold) ||
+			(right  && lx >= output_box->x + output_box->width - 1 - threshold)) {
 		if (output->fullscreen_view) {
 			output->force_shell_reveal = true;
 			phoc_output_damage_whole(output);
@@ -194,7 +191,7 @@ static void roots_passthrough_cursor(struct roots_cursor *cursor,
 		return;
 	}
 
-	if (cursor->cursor_client != client) {
+	if (cursor->cursor_client != client || !client) {
 		roots_seat_maybe_set_cursor (seat, NULL);
 		cursor->cursor_client = client;
 	}
@@ -244,6 +241,8 @@ void roots_cursor_update_focus(struct roots_cursor *cursor) {
 
 void roots_cursor_update_position(struct roots_cursor *cursor,
 		uint32_t time) {
+	PhocServer *server = phoc_server_get_default ();
+	PhocDesktop *desktop = server->desktop;
 	struct roots_seat *seat = cursor->seat;
 	struct roots_view *view;
 	switch (cursor->mode) {
@@ -253,15 +252,34 @@ void roots_cursor_update_position(struct roots_cursor *cursor,
 	case ROOTS_CURSOR_MOVE:
 		view = roots_seat_get_focus(seat);
 		if (view != NULL) {
+			struct wlr_box geom;
+			view_get_geometry(view, &geom);
 			double dx = cursor->cursor->x - cursor->offs_x;
 			double dy = cursor->cursor->y - cursor->offs_y;
-			view_move(view, cursor->view_x + dx,
-				cursor->view_y + dy);
+
+			struct wlr_output *wlr_output = wlr_output_layout_output_at(desktop->layout, cursor->cursor->x, cursor->cursor->y);
+			struct wlr_box *output_box = wlr_output_layout_get_box(desktop->layout, wlr_output);
+
+			bool output_is_landscape = output_box->width > output_box->height;
+
+			if (cursor->cursor->y < output_box->y + PHOC_EDGE_SNAP_THRESHOLD) {
+				view_maximize(view, wlr_output);
+			} else if (output_is_landscape && cursor->cursor->x < output_box->x + PHOC_EDGE_SNAP_THRESHOLD) {
+				view_tile(view, PHOC_VIEW_TILE_LEFT, wlr_output);
+			} else if (output_is_landscape && cursor->cursor->x > output_box->x + output_box->width - PHOC_EDGE_SNAP_THRESHOLD) {
+				view_tile(view, PHOC_VIEW_TILE_RIGHT, wlr_output);
+			} else {
+				view_restore(view);
+				view_move(view, cursor->view_x + dx - geom.x * view->scale,
+				          cursor->view_y + dy - geom.y * view->scale);
+			}
 		}
 		break;
 	case ROOTS_CURSOR_RESIZE:
 		view = roots_seat_get_focus(seat);
 		if (view != NULL) {
+			struct wlr_box geom;
+			view_get_geometry(view, &geom);
 			double dx = cursor->cursor->x - cursor->offs_x;
 			double dy = cursor->cursor->y - cursor->offs_y;
 			double x = view->box.x;
@@ -269,7 +287,7 @@ void roots_cursor_update_position(struct roots_cursor *cursor,
 			int width = cursor->view_width;
 			int height = cursor->view_height;
 			if (cursor->resize_edges & WLR_EDGE_TOP) {
-				y = cursor->view_y + dy;
+				y = cursor->view_y + dy - geom.y * view->scale;
 				height -= dy;
 				if (height < 1) {
 					y += height;
@@ -278,7 +296,7 @@ void roots_cursor_update_position(struct roots_cursor *cursor,
 				height += dy;
 			}
 			if (cursor->resize_edges & WLR_EDGE_LEFT) {
-				x = cursor->view_x + dx;
+				x = cursor->view_x + dx - geom.x * view->scale;
 				width -= dx;
 				if (width < 1) {
 					x += width;
@@ -464,6 +482,12 @@ void roots_cursor_handle_touch_down(struct roots_cursor *cursor,
 	wlr_cursor_absolute_to_layout_coords(cursor->cursor, event->device,
 		event->x, event->y, &lx, &ly);
 
+	if (seat->touch_id == -1 && cursor->mode == ROOTS_CURSOR_PASSTHROUGH) {
+		seat->touch_id = event->touch_id;
+		seat->touch_x = lx;
+		seat->touch_y = ly;
+	}
+
 	double sx, sy;
 	struct roots_view *view;
 	struct wlr_surface *surface = phoc_desktop_surface_at(
@@ -511,10 +535,20 @@ void roots_cursor_handle_touch_up(struct roots_cursor *cursor,
 		struct wlr_event_touch_up *event) {
 	struct wlr_touch_point *point =
 		wlr_seat_touch_get_point(cursor->seat->seat, event->touch_id);
+
+	if (cursor->seat->touch_id == event->touch_id) {
+		cursor->seat->touch_id = -1;
+	}
+
 	if (!point) {
 		return;
 	}
 
+	if (cursor->mode != ROOTS_CURSOR_PASSTHROUGH) {
+		cursor->mode = ROOTS_CURSOR_PASSTHROUGH;
+		roots_cursor_update_focus(cursor);
+	}
+
 	wlr_seat_touch_notify_up(cursor->seat->seat, event->time_msec,
 		event->touch_id);
 }
@@ -601,6 +635,11 @@ void roots_cursor_handle_touch_motion(struct roots_cursor *cursor,
 	if (event->touch_id == cursor->seat->touch_id) {
 		cursor->seat->touch_x = lx;
 		cursor->seat->touch_y = ly;
+
+		if (cursor->mode != ROOTS_CURSOR_PASSTHROUGH) {
+			wlr_cursor_warp(cursor->cursor, NULL, lx, ly);
+			roots_cursor_update_position(cursor, event->time_msec);
+		}
 	}
 }
 
