@@ -15,7 +15,6 @@
 #include <wlr/types/wlr_export_dmabuf_v1.h>
 #include <wlr/types/wlr_gamma_control_v1.h>
 #include <wlr/types/wlr_gtk_primary_selection.h>
-#include <wlr/types/wlr_idle_inhibit_v1.h>
 #include <wlr/types/wlr_idle.h>
 #include <wlr/types/wlr_input_inhibitor.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
@@ -547,16 +546,6 @@ phoc_desktop_constructed (GObject *object)
 #ifdef PHOC_XWAYLAND
   const char *cursor_default = ROOTS_XCURSOR_DEFAULT;
 #endif
-  struct roots_cursor_config *cc =
-    roots_config_get_cursor(config, ROOTS_CONFIG_DEFAULT_SEAT_NAME);
-  if (cc != NULL) {
-    cursor_theme = cc->theme;
-#ifdef PHOC_XWAYLAND
-    if (cc->default_image != NULL) {
-      cursor_default = cc->default_image;
-    }
-#endif
-  }
 
   char cursor_size_fmt[16];
   snprintf(cursor_size_fmt, sizeof(cursor_size_fmt),
@@ -610,7 +599,6 @@ phoc_desktop_constructed (GObject *object)
   wlr_server_decoration_manager_set_default_mode(self->server_decoration_manager,
 						 WLR_SERVER_DECORATION_MANAGER_MODE_CLIENT);
   self->idle = wlr_idle_create(server->wl_display);
-  self->idle_inhibit = wlr_idle_inhibit_v1_create(server->wl_display);
   self->primary_selection_device_manager =
     wlr_gtk_primary_selection_device_manager_create(server->wl_display);
   self->input_inhibit =
@@ -785,11 +773,16 @@ phoc_desktop_set_auto_maximize (PhocDesktop *self, gboolean enable)
   self->maximize = enable;
 
   /* Disabling auto-maximize leaves all views in their current position */
-  if (!enable)
+  if (!enable) {
+    wl_list_for_each (view, &self->views, link)
+      view_appear_activated (view, phoc_input_view_has_focus (phoc_server_get_default()->input, view));
     return;
+  }
 
-  wl_list_for_each (view, &self->views, link)
+  wl_list_for_each (view, &self->views, link) {
     view_auto_maximize (view);
+    view_appear_activated (view, true);
+  }
 }
 
 gboolean
