@@ -281,9 +281,9 @@ void arrange_layers(PhocOutput *output) {
 	struct roots_view *view;
 	wl_list_for_each(view, &output->desktop->views, link) {
 		if (view_is_maximized(view)) {
-			view_arrange_maximized(view);
+			view_arrange_maximized(view, NULL);
 		} else if (view_is_tiled(view)) {
-			view_arrange_tiled(view);
+			view_arrange_tiled(view, NULL);
 		} else if (output->desktop->maximize) {
 			view_center(view);
 		}
