@@ -282,15 +282,19 @@ static bool scan_out_fullscreen_view(PhocOutput *output) {
 		return false;
 	}
 
-	wlr_presentation_surface_sampled(output->desktop->presentation, surface);
-
 #if WLR_VERSION_MAJOR == 0 && WLR_VERSION_MINOR < 11
 	if (!wlr_output_attach_buffer(wlr_output, surface->buffer)) {
 		return false;
 	}
 #else
 	wlr_output_attach_buffer(wlr_output, &surface->buffer->base);
+	if (!wlr_output_test(wlr_output)) {
+		return false;
+	}
 #endif
+
+	wlr_presentation_surface_sampled_on_output(output->desktop->presentation, surface, output->wlr_output);
+
 	return wlr_output_commit(wlr_output);
 }
 
