@@ -22,6 +22,7 @@
 #include <wlr/version.h>
 #include "cursor.h"
 #include "keyboard.h"
+#include "pointer.h"
 #include "seat.h"
 #include "text_input.h"
 #include "touch.h"
@@ -484,7 +485,7 @@ void roots_seat_configure_cursor(struct roots_seat *seat) {
 	PhocDesktop *desktop = server->desktop;
 	struct wlr_cursor *cursor = seat->cursor->cursor;
 
-	struct roots_pointer *pointer;
+	PhocPointer *pointer;
 	PhocTouch *touch;
 	struct roots_tablet *tablet;
 	PhocOutput *output;
@@ -527,9 +528,7 @@ void roots_seat_configure_cursor(struct roots_seat *seat) {
 static void roots_seat_init_cursor(struct roots_seat *seat) {
 	PhocServer *server = phoc_server_get_default ();
 	seat->cursor = roots_cursor_create(seat);
-	if (!seat->cursor) {
-		return;
-	}
+
 	seat->cursor->seat = seat;
 	struct wlr_cursor *wlr_cursor = seat->cursor->cursor;
 	PhocDesktop *desktop = server->desktop;
@@ -763,6 +762,8 @@ static void roots_seat_handle_destroy(struct wl_listener *listener,
 	// TODO: probably more to be freed here
 	wl_list_remove(&seat->destroy.link);
 
+	roots_input_method_relay_destroy(&seat->im_relay);
+
 	struct roots_seat_view *view, *nview;
 	wl_list_for_each_safe(view, nview, &seat->views, link) {
 		seat_view_destroy(view);
@@ -791,6 +792,8 @@ struct roots_seat *roots_seat_create(PhocInput *input, char *name) {
 
 	seat->input = input;
 
+	seat->touch_id = -1;
+
 	seat->seat = wlr_seat_create(server->wl_display, name);
 	if (!seat->seat) {
 		free(seat);
@@ -875,29 +878,21 @@ static void seat_add_keyboard(struct roots_seat *seat,
 }
 
 static void handle_pointer_destroy(struct wl_listener *listener, void *data) {
-	struct roots_pointer *pointer =
-		wl_container_of(listener, pointer, device_destroy);
+	PhocPointer *pointer = wl_container_of(listener, pointer, device_destroy);
 	struct roots_seat *seat = pointer->seat;
 
 	wl_list_remove(&pointer->link);
 	wlr_cursor_detach_input_device(seat->cursor->cursor, pointer->device);
 	wl_list_remove(&pointer->device_destroy.link);
-	free(pointer);
+	g_object_unref (pointer);
 
 	seat_update_capabilities(seat);
 }
 
 static void seat_add_pointer(struct roots_seat *seat,
 		struct wlr_input_device *device) {
-	struct roots_pointer *pointer = calloc(1, sizeof(struct roots_pointer));
-	if (!pointer) {
-		wlr_log(WLR_ERROR, "could not allocate pointer for seat");
-		return;
-	}
+	PhocPointer *pointer = phoc_pointer_new (device, seat);
 
-	device->data = pointer;
-	pointer->device = device;
-	pointer->seat = seat;
 	wl_list_insert(&seat->pointers, &pointer->link);
 
 	pointer->device_destroy.notify = handle_pointer_destroy;
@@ -1207,21 +1202,12 @@ void roots_seat_add_device(struct roots_seat *seat,
 void roots_seat_configure_xcursor(struct roots_seat *seat) {
 	PhocServer *server = phoc_server_get_default ();
 	const char *cursor_theme = NULL;
-	struct roots_cursor_config *cc =
-		roots_config_get_cursor(seat->input->config, seat->seat->name);
-	if (cc != NULL) {
-		cursor_theme = cc->theme;
-		if (cc->default_image != NULL) {
-			seat->cursor->default_xcursor = cc->default_image;
-		}
-	}
 
 	if (!seat->cursor->xcursor_manager) {
 		seat->cursor->xcursor_manager =
 			wlr_xcursor_manager_create(cursor_theme, ROOTS_XCURSOR_SIZE);
 		if (seat->cursor->xcursor_manager == NULL) {
-			wlr_log(WLR_ERROR, "Cannot create XCursor manager for theme %s",
-					cursor_theme);
+			wlr_log(WLR_ERROR, "Cannot create XCursor manager for theme");
 			return;
 		}
 	}
@@ -1488,10 +1474,7 @@ void roots_seat_set_focus(struct roots_seat *seat, struct roots_view *view) {
 			NULL, 0, NULL);
 	}
 
-	if (seat->cursor) {
-		roots_cursor_update_focus(seat->cursor);
-	}
-
+	roots_cursor_update_focus(seat->cursor);
 	roots_input_method_relay_set_focus(&seat->im_relay, view->wlr_surface);
 }
 
@@ -1538,10 +1521,7 @@ void roots_seat_set_focus_layer(struct roots_seat *seat,
 			NULL, 0, NULL);
 	}
 
-	if (seat->cursor) {
-		roots_cursor_update_focus(seat->cursor);
-	}
-
+	roots_cursor_update_focus(seat->cursor);
 	roots_input_method_relay_set_focus(&seat->im_relay, layer->surface);
 }
 
@@ -1617,8 +1597,13 @@ void roots_seat_begin_move(struct roots_seat *seat, struct roots_view *view) {
 
 	struct roots_cursor *cursor = seat->cursor;
 	cursor->mode = ROOTS_CURSOR_MOVE;
+	if (seat->touch_id != -1) {
+		wlr_cursor_warp(cursor->cursor, NULL, seat->touch_x, seat->touch_y);
+	}
 	cursor->offs_x = cursor->cursor->x;
 	cursor->offs_y = cursor->cursor->y;
+	struct wlr_box geom;
+	view_get_geometry(view, &geom);
 	if (view_is_maximized(view) || view_is_tiled(view)) {
 		// calculate normalized (0..1) position of cursor in maximized window
 		// and make it stay the same after restoring saved size
@@ -1630,8 +1615,8 @@ void roots_seat_begin_move(struct roots_seat *seat, struct roots_view *view) {
 		view->saved.y = cursor->view_y;
 		view_restore(view);
 	} else {
-		cursor->view_x = view->box.x;
-		cursor->view_y = view->box.y;
+		cursor->view_x = view->box.x + geom.x * view->scale;
+		cursor->view_y = view->box.y + geom.y * view->scale;
 	}
 	wlr_seat_pointer_clear_focus(seat->seat);
 
@@ -1645,18 +1630,23 @@ void roots_seat_begin_resize(struct roots_seat *seat, struct roots_view *view,
 
 	struct roots_cursor *cursor = seat->cursor;
 	cursor->mode = ROOTS_CURSOR_RESIZE;
+	if (seat->touch_id != -1) {
+		wlr_cursor_warp(cursor->cursor, NULL, seat->touch_x, seat->touch_y);
+	}
 	cursor->offs_x = cursor->cursor->x;
 	cursor->offs_y = cursor->cursor->y;
+	struct wlr_box geom;
+	view_get_geometry(view, &geom);
 	if (view_is_maximized(view) || view_is_tiled(view)) {
-		view->saved.x = view->box.x;
-		view->saved.y = view->box.y;
+		view->saved.x = view->box.x + geom.x * view->scale;
+		view->saved.y = view->box.y + geom.y * view->scale;
 		view->saved.width = view->box.width;
 		view->saved.height = view->box.height;
 		view_restore(view);
 	}
 
-	cursor->view_x = view->box.x;
-	cursor->view_y = view->box.y;
+	cursor->view_x = view->box.x + geom.x * view->scale;
+	cursor->view_y = view->box.y + geom.y * view->scale;
 	struct wlr_box box;
 	view_get_box(view, &box);
 	cursor->view_width = box.width;
@@ -1690,6 +1680,7 @@ void roots_seat_end_compositor_grab(struct roots_seat *seat) {
 	}
 
 	cursor->mode = ROOTS_CURSOR_PASSTHROUGH;
+	roots_cursor_update_focus(seat->cursor);
 }
 
 struct roots_seat *input_last_active_seat(PhocInput *input) {
