@@ -199,8 +199,10 @@ view_save(struct roots_view *view)
     return;
 
   /* backup window state */
-  view->saved.x = view->box.x;
-  view->saved.y = view->box.y;
+  struct wlr_box geom;
+  view_get_geometry(view, &geom);
+  view->saved.x = view->box.x + geom.x * view->scale;
+  view->saved.y = view->box.y + geom.y * view->scale;
   view->saved.width = view->box.width;
   view->saved.height = view->box.height;
 }
@@ -223,9 +225,16 @@ void view_move(struct roots_view *view, double x, double y) {
 	view_update_output(view, &before);
 }
 
+void
+view_appear_activated (struct roots_view *view, bool activated)
+{
+  if (view->impl->activate)
+    view->impl->activate (view, activated);
+}
+
 void view_activate(struct roots_view *view, bool activate) {
-	if (view->impl->activate) {
-		view->impl->activate(view, activate);
+	if (!view->desktop->maximize) {
+		view_appear_activated(view, activate);
 	}
 
 	if (view->toplevel_handle) {
@@ -286,15 +295,15 @@ static struct wlr_output *view_get_output(struct roots_view *view) {
 		output_y);
 }
 
-void view_arrange_maximized(struct roots_view *view) {
-	if (view->fullscreen_output != NULL) {
+void view_arrange_maximized(struct roots_view *view, struct wlr_output *output) {
+	if (view->fullscreen_output)
 		return;
-	}
 
-	struct wlr_output *output = view_get_output(view);
-	if (!output) {
+	if (!output)
+		output = view_get_output(view);
+
+	if (!output)
 		return;
-	}
 
 	PhocOutput *phoc_output = output->data;
 	struct wlr_box *output_box =
@@ -308,13 +317,15 @@ void view_arrange_maximized(struct roots_view *view) {
 }
 
 void
-view_arrange_tiled (struct roots_view *view)
+view_arrange_tiled (struct roots_view *view, struct wlr_output *output)
 {
-  struct wlr_output *output = view_get_output (view);
-  if (!output)
+  if (view->fullscreen_output)
     return;
 
-  if (view->fullscreen_output)
+  if (!output)
+    output = view_get_output(view);
+
+  if (!output)
     return;
 
   PhocOutput *phoc_output = output->data;
@@ -359,8 +370,12 @@ want_auto_maximize(struct roots_view *view) {
   return false;
 }
 
-void view_maximize(struct roots_view *view) {
-	if (view_is_maximized (view) || view->fullscreen_output != NULL) {
+void view_maximize(struct roots_view *view, struct wlr_output *output) {
+	if (view_is_maximized (view) && view_get_output(view) == output) {
+		return;
+	}
+
+	if (view->fullscreen_output != NULL) {
 		return;
 	}
 
@@ -375,7 +390,7 @@ void view_maximize(struct roots_view *view) {
 	view_save (view);
 
 	view->state = PHOC_VIEW_STATE_MAXIMIZED;
-	view_arrange_maximized(view);
+	view_arrange_maximized(view, output);
 }
 
 /*
@@ -385,7 +400,7 @@ void
 view_auto_maximize(struct roots_view *view)
 {
   if (want_auto_maximize (view))
-    view_maximize (view);
+    view_maximize (view, NULL);
 }
 
 void
@@ -397,8 +412,11 @@ view_restore(struct roots_view *view)
   if (want_auto_maximize (view))
     return;
 
+  struct wlr_box geom;
+  view_get_geometry(view, &geom);
+
   view->state = PHOC_VIEW_STATE_NORMAL;
-  view_move_resize (view, view->saved.x, view->saved.y,
+  view_move_resize (view, view->saved.x - geom.x * view->scale, view->saved.y - geom.y * view->scale,
                     view->saved.width, view->saved.height);
 
   if (view->toplevel_handle)
@@ -425,6 +443,9 @@ void view_set_fullscreen(struct roots_view *view, bool fullscreen,
 		}
 	}
 
+	struct wlr_box view_geom;
+	view_get_geometry(view, &view_geom);
+
 	if (fullscreen) {
 		if (output == NULL) {
 			output = view_get_output(view);
@@ -455,17 +476,18 @@ void view_set_fullscreen(struct roots_view *view, bool fullscreen,
 	}
 
 	if (was_fullscreen && !fullscreen) {
-		phoc_output_damage_whole(view->fullscreen_output);
-
+		PhocOutput *phoc_output = view->fullscreen_output;
 		view->fullscreen_output->fullscreen_view = NULL;
 		view->fullscreen_output = NULL;
 
+		phoc_output_damage_whole(phoc_output);
+
 		if (view->state == PHOC_VIEW_STATE_MAXIMIZED) {
-			view_arrange_maximized (view);
+			view_arrange_maximized (view, phoc_output->wlr_output);
 		} else if (view->state == PHOC_VIEW_STATE_TILED) {
-			view_arrange_tiled (view);
+			view_arrange_tiled (view, phoc_output->wlr_output);
 		} else {
-			view_move_resize(view, view->saved.x, view->saved.y,
+			view_move_resize(view, view->saved.x - view_geom.x * view->scale, view->saved.y - view_geom.y * view->scale,
 			                 view->saved.width, view->saved.height);
 		}
 
@@ -514,19 +536,20 @@ view_move_to_next_output (struct roots_view *view, enum wlr_direction direction)
 
   view->saved.x = x;
   view->saved.y = y;
-  view_move(view, x, y);
 
   if (view_is_maximized (view)) {
-    view_arrange_maximized (view);
+    view_arrange_maximized (view, new_output);
   } else if (view_is_tiled (view)) {
-    view_arrange_tiled (view);
+    view_arrange_tiled (view, new_output);
+  } else {
+    view_move(view, x, y);
   }
 
   return true;
 }
 
 void
-view_tile(struct roots_view *view, PhocViewTileDirection direction)
+view_tile(struct roots_view *view, PhocViewTileDirection direction, struct wlr_output *output)
 {
   if (view->fullscreen_output)
     return;
@@ -539,7 +562,7 @@ view_tile(struct roots_view *view, PhocViewTileDirection direction)
 
   view->state = PHOC_VIEW_STATE_TILED;
   view->tile_direction = direction;
-  view_arrange_tiled (view);
+  view_arrange_tiled (view, output);
 }
 
 bool view_center(struct roots_view *view) {
@@ -772,9 +795,9 @@ static void view_update_scale(struct roots_view *view) {
 	}
 	if (view->scale != oldscale) {
 		if (view_is_maximized(view)) {
-			view_arrange_maximized(view);
+			view_arrange_maximized(view, NULL);
 		} else if (view_is_tiled(view)) {
-			view_arrange_tiled(view);
+			view_arrange_tiled(view, NULL);
 		} else {
 			view_center(view);
 		}
@@ -797,14 +820,19 @@ void view_map(struct roots_view *view, struct wlr_surface *surface) {
 	wl_signal_add(&view->wlr_surface->events.new_subsurface,
 		&view->new_subsurface);
 
-	if (view->desktop->maximize && !wl_list_empty(&view->desktop->views)) {
-		// mapping a new stack may make the old stack disappear, so damage its area
-		struct roots_view *top_view = wl_container_of(view->desktop->views.next, view, link);
-		while (top_view) {
-			view_damage_whole(top_view);
-			top_view = top_view->parent;
+	if (view->desktop->maximize) {
+		view_appear_activated(view, true);
+
+		if (!wl_list_empty(&view->desktop->views)) {
+			// mapping a new stack may make the old stack disappear, so damage its area
+			struct roots_view *top_view = wl_container_of(view->desktop->views.next, view, link);
+			while (top_view) {
+				view_damage_whole(top_view);
+				top_view = top_view->parent;
+			}
 		}
 	}
+
 	wl_list_insert(&view->desktop->views, &view->link);
 	view_damage_whole(view);
 	phoc_input_update_cursor_focus(server->input);
@@ -994,7 +1022,7 @@ static void handle_toplevel_handle_request_maximize(struct wl_listener *listener
 			toplevel_handle_request_maximize);
 	struct wlr_foreign_toplevel_handle_v1_maximized_event *event = data;
 	if (event->maximized) {
-		view_maximize(view);
+		view_maximize(view, NULL);
 	} else {
 		view_restore(view);
 	}
